/********************************************************************************
** Form generated from reading UI file 'trackui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TRACKUI_H
#define UI_TRACKUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_TrackWidget
{
public:
    QGridLayout *gridLayout_2;
    QGridLayout *gridLayout;
    QLabel *label;
    QCheckBox *titleCheck;
    QLineEdit *titleText;
    QCheckBox *moveCheck;
    QHBoxLayout *horizontalLayout_9;
    QSpinBox *weeksSpin;
    QLabel *weeksLabel;
    QHBoxLayout *horizontalLayout_5;
    QSpinBox *daysSpin;
    QLabel *daysLabel;
    QHBoxLayout *horizontalLayout_4;
    QSpinBox *hoursSpin;
    QLabel *hoursLabel;
    QHBoxLayout *horizontalLayout_2;
    QSpinBox *minsSpin;
    QLabel *minsLabel;
    QHBoxLayout *horizontalLayout;
    QSpinBox *secsSpin;
    QLabel *secsLabel;
    QCheckBox *startCheck;
    QDateTimeEdit *startEdit;
    QCheckBox *stopCheck;
    QDateTimeEdit *stopEdit;
    QCheckBox *packCheck;
    QCheckBox *mergeCheck;
    QRadioButton *splitDateCheck;
    QRadioButton *splitTimeCheck;
    QHBoxLayout *horizontalLayout_6;
    QSpinBox *splitTimeSpin;
    QComboBox *splitTimeCombo;
    QRadioButton *splitDistanceCheck;
    QHBoxLayout *horizontalLayout_7;
    QSpinBox *splitDistSpin;
    QComboBox *splitDistCombo;
    QHBoxLayout *horizontalLayout_8;
    QCheckBox *GPSFixesCheck;
    QComboBox *GPSFixesCombo;
    QSpacerItem *horizontalSpacer_2;
    QCheckBox *speedCheck;
    QCheckBox *courseCheck;
    QRadioButton *localTime;
    QRadioButton *utc;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer;
    QButtonGroup *buttonGroup;

    void setupUi(QWidget *TrackWidget)
    {
        if (TrackWidget->objectName().isEmpty())
            TrackWidget->setObjectName("TrackWidget");
        TrackWidget->resize(675, 270);
        gridLayout_2 = new QGridLayout(TrackWidget);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(TrackWidget);
        label->setObjectName("label");
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        label->setFont(font);

        gridLayout->addWidget(label, 0, 0, 1, 2);

        titleCheck = new QCheckBox(TrackWidget);
        titleCheck->setObjectName("titleCheck");

        gridLayout->addWidget(titleCheck, 1, 0, 1, 1);

        titleText = new QLineEdit(TrackWidget);
        titleText->setObjectName("titleText");

        gridLayout->addWidget(titleText, 1, 1, 1, 4);

        moveCheck = new QCheckBox(TrackWidget);
        moveCheck->setObjectName("moveCheck");

        gridLayout->addWidget(moveCheck, 2, 0, 1, 1);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        weeksSpin = new QSpinBox(TrackWidget);
        weeksSpin->setObjectName("weeksSpin");
        weeksSpin->setAccelerated(true);
        weeksSpin->setMinimum(-10000);
        weeksSpin->setMaximum(10000);

        horizontalLayout_9->addWidget(weeksSpin);

        weeksLabel = new QLabel(TrackWidget);
        weeksLabel->setObjectName("weeksLabel");

        horizontalLayout_9->addWidget(weeksLabel);


        gridLayout->addLayout(horizontalLayout_9, 2, 1, 1, 1);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        daysSpin = new QSpinBox(TrackWidget);
        daysSpin->setObjectName("daysSpin");
        daysSpin->setAccelerated(true);
        daysSpin->setMinimum(-100000);
        daysSpin->setMaximum(100000);

        horizontalLayout_5->addWidget(daysSpin);

        daysLabel = new QLabel(TrackWidget);
        daysLabel->setObjectName("daysLabel");

        horizontalLayout_5->addWidget(daysLabel);


        gridLayout->addLayout(horizontalLayout_5, 2, 2, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        hoursSpin = new QSpinBox(TrackWidget);
        hoursSpin->setObjectName("hoursSpin");
        hoursSpin->setAccelerated(true);
        hoursSpin->setMinimum(-100000);
        hoursSpin->setMaximum(100000);

        horizontalLayout_4->addWidget(hoursSpin);

        hoursLabel = new QLabel(TrackWidget);
        hoursLabel->setObjectName("hoursLabel");

        horizontalLayout_4->addWidget(hoursLabel);


        gridLayout->addLayout(horizontalLayout_4, 2, 3, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        minsSpin = new QSpinBox(TrackWidget);
        minsSpin->setObjectName("minsSpin");
        minsSpin->setAccelerated(true);
        minsSpin->setMinimum(-100000);
        minsSpin->setMaximum(100000);

        horizontalLayout_2->addWidget(minsSpin);

        minsLabel = new QLabel(TrackWidget);
        minsLabel->setObjectName("minsLabel");

        horizontalLayout_2->addWidget(minsLabel);


        gridLayout->addLayout(horizontalLayout_2, 2, 4, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        secsSpin = new QSpinBox(TrackWidget);
        secsSpin->setObjectName("secsSpin");
        secsSpin->setAccelerated(true);
        secsSpin->setMinimum(-100000);
        secsSpin->setMaximum(100000);

        horizontalLayout->addWidget(secsSpin);

        secsLabel = new QLabel(TrackWidget);
        secsLabel->setObjectName("secsLabel");

        horizontalLayout->addWidget(secsLabel);


        gridLayout->addLayout(horizontalLayout, 2, 5, 1, 1);

        startCheck = new QCheckBox(TrackWidget);
        startCheck->setObjectName("startCheck");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(startCheck->sizePolicy().hasHeightForWidth());
        startCheck->setSizePolicy(sizePolicy);

        gridLayout->addWidget(startCheck, 3, 0, 1, 1);

        startEdit = new QDateTimeEdit(TrackWidget);
        startEdit->setObjectName("startEdit");

        gridLayout->addWidget(startEdit, 3, 1, 1, 2);

        stopCheck = new QCheckBox(TrackWidget);
        stopCheck->setObjectName("stopCheck");
        sizePolicy.setHeightForWidth(stopCheck->sizePolicy().hasHeightForWidth());
        stopCheck->setSizePolicy(sizePolicy);

        gridLayout->addWidget(stopCheck, 4, 0, 1, 1);

        stopEdit = new QDateTimeEdit(TrackWidget);
        stopEdit->setObjectName("stopEdit");

        gridLayout->addWidget(stopEdit, 4, 1, 1, 2);

        packCheck = new QCheckBox(TrackWidget);
        packCheck->setObjectName("packCheck");

        gridLayout->addWidget(packCheck, 5, 0, 1, 1);

        mergeCheck = new QCheckBox(TrackWidget);
        mergeCheck->setObjectName("mergeCheck");

        gridLayout->addWidget(mergeCheck, 5, 1, 1, 1);

        splitDateCheck = new QRadioButton(TrackWidget);
        splitDateCheck->setObjectName("splitDateCheck");

        gridLayout->addWidget(splitDateCheck, 6, 0, 1, 2);

        splitTimeCheck = new QRadioButton(TrackWidget);
        splitTimeCheck->setObjectName("splitTimeCheck");
        splitTimeCheck->setAutoExclusive(false);

        gridLayout->addWidget(splitTimeCheck, 6, 2, 1, 1);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        splitTimeSpin = new QSpinBox(TrackWidget);
        splitTimeSpin->setObjectName("splitTimeSpin");
        splitTimeSpin->setMaximum(500);

        horizontalLayout_6->addWidget(splitTimeSpin);

        splitTimeCombo = new QComboBox(TrackWidget);
        splitTimeCombo->addItem(QString());
        splitTimeCombo->addItem(QString());
        splitTimeCombo->addItem(QString());
        splitTimeCombo->setObjectName("splitTimeCombo");

        horizontalLayout_6->addWidget(splitTimeCombo);


        gridLayout->addLayout(horizontalLayout_6, 6, 3, 1, 1);

        splitDistanceCheck = new QRadioButton(TrackWidget);
        splitDistanceCheck->setObjectName("splitDistanceCheck");
        splitDistanceCheck->setAutoExclusive(false);

        gridLayout->addWidget(splitDistanceCheck, 6, 4, 1, 1);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        splitDistSpin = new QSpinBox(TrackWidget);
        splitDistSpin->setObjectName("splitDistSpin");
        splitDistSpin->setMaximum(5280);

        horizontalLayout_7->addWidget(splitDistSpin);

        splitDistCombo = new QComboBox(TrackWidget);
        splitDistCombo->addItem(QString());
        splitDistCombo->addItem(QString());
        splitDistCombo->addItem(QString());
        splitDistCombo->addItem(QString());
        splitDistCombo->setObjectName("splitDistCombo");

        horizontalLayout_7->addWidget(splitDistCombo);


        gridLayout->addLayout(horizontalLayout_7, 6, 5, 1, 1);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        GPSFixesCheck = new QCheckBox(TrackWidget);
        GPSFixesCheck->setObjectName("GPSFixesCheck");
        sizePolicy.setHeightForWidth(GPSFixesCheck->sizePolicy().hasHeightForWidth());
        GPSFixesCheck->setSizePolicy(sizePolicy);

        horizontalLayout_8->addWidget(GPSFixesCheck);

        GPSFixesCombo = new QComboBox(TrackWidget);
        GPSFixesCombo->addItem(QString());
        GPSFixesCombo->addItem(QString());
        GPSFixesCombo->addItem(QString());
        GPSFixesCombo->addItem(QString());
        GPSFixesCombo->addItem(QString());
        GPSFixesCombo->setObjectName("GPSFixesCombo");

        horizontalLayout_8->addWidget(GPSFixesCombo);

        horizontalSpacer_2 = new QSpacerItem(2, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_2);


        gridLayout->addLayout(horizontalLayout_8, 7, 0, 1, 2);

        speedCheck = new QCheckBox(TrackWidget);
        speedCheck->setObjectName("speedCheck");

        gridLayout->addWidget(speedCheck, 7, 4, 1, 1);

        courseCheck = new QCheckBox(TrackWidget);
        courseCheck->setObjectName("courseCheck");

        gridLayout->addWidget(courseCheck, 7, 5, 1, 1);

        localTime = new QRadioButton(TrackWidget);
        buttonGroup = new QButtonGroup(TrackWidget);
        buttonGroup->setObjectName("buttonGroup");
        buttonGroup->addButton(localTime);
        localTime->setObjectName("localTime");

        gridLayout->addWidget(localTime, 3, 3, 2, 1);

        utc = new QRadioButton(TrackWidget);
        buttonGroup->addButton(utc);
        utc->setObjectName("utc");

        gridLayout->addWidget(utc, 3, 4, 2, 1);


        gridLayout_2->addLayout(gridLayout, 0, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(1, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 0, 1, 1, 1);

        verticalSpacer = new QSpacerItem(10, 2, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 1, 0, 1, 1);

        QWidget::setTabOrder(titleCheck, titleText);
        QWidget::setTabOrder(titleText, moveCheck);
        QWidget::setTabOrder(moveCheck, weeksSpin);
        QWidget::setTabOrder(weeksSpin, daysSpin);
        QWidget::setTabOrder(daysSpin, hoursSpin);
        QWidget::setTabOrder(hoursSpin, minsSpin);
        QWidget::setTabOrder(minsSpin, secsSpin);
        QWidget::setTabOrder(secsSpin, startCheck);
        QWidget::setTabOrder(startCheck, startEdit);
        QWidget::setTabOrder(startEdit, localTime);
        QWidget::setTabOrder(localTime, utc);
        QWidget::setTabOrder(utc, stopCheck);
        QWidget::setTabOrder(stopCheck, stopEdit);
        QWidget::setTabOrder(stopEdit, packCheck);
        QWidget::setTabOrder(packCheck, mergeCheck);
        QWidget::setTabOrder(mergeCheck, splitDateCheck);
        QWidget::setTabOrder(splitDateCheck, splitTimeCheck);
        QWidget::setTabOrder(splitTimeCheck, splitTimeSpin);
        QWidget::setTabOrder(splitTimeSpin, splitTimeCombo);
        QWidget::setTabOrder(splitTimeCombo, splitDistanceCheck);
        QWidget::setTabOrder(splitDistanceCheck, splitDistSpin);
        QWidget::setTabOrder(splitDistSpin, splitDistCombo);
        QWidget::setTabOrder(splitDistCombo, GPSFixesCheck);
        QWidget::setTabOrder(GPSFixesCheck, GPSFixesCombo);
        QWidget::setTabOrder(GPSFixesCombo, speedCheck);
        QWidget::setTabOrder(speedCheck, courseCheck);

        retranslateUi(TrackWidget);

        QMetaObject::connectSlotsByName(TrackWidget);
    } // setupUi

    void retranslateUi(QWidget *TrackWidget)
    {
        TrackWidget->setWindowTitle(QCoreApplication::translate("TrackWidget", "Track Filter Options", nullptr));
#if QT_CONFIG(whatsthis)
        TrackWidget->setWhatsThis(QCoreApplication::translate("TrackWidget", "This filter performs various operations on track data. ", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        label->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
        label->setText(QCoreApplication::translate("TrackWidget", "Track Filters", nullptr));
#if QT_CONFIG(tooltip)
        titleCheck->setToolTip(QCoreApplication::translate("TrackWidget", "Basic title for track", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        titleCheck->setWhatsThis(QCoreApplication::translate("TrackWidget", "Basic title for new track(s). \n"
"This option specifies a title for tracks generated by the track filter. By default, the title of the new track is composed of the start time of the track appended to this value. \n"
"If this value contains a percent (%) character, it is treated as a format string for the POSIX strftime function, allowing custom time-based track names. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        titleCheck->setText(QCoreApplication::translate("TrackWidget", "Title", nullptr));
#if QT_CONFIG(tooltip)
        titleText->setToolTip(QCoreApplication::translate("TrackWidget", "the title of the new track is composed of the start time of the track appended to this value. ", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        moveCheck->setToolTip(QCoreApplication::translate("TrackWidget", "Correct track point timestamps by specified amount", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        moveCheck->setWhatsThis(QCoreApplication::translate("TrackWidget", "Correct trackpoint timestamps by a delta. \n"
"This option changes the time of all trackpoints. This might be useful if your track must be moved by one or more hours because of an incorrect time zone. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        moveCheck->setText(QCoreApplication::translate("TrackWidget", "Move", nullptr));
        weeksLabel->setText(QCoreApplication::translate("TrackWidget", "weeks", nullptr));
        daysLabel->setText(QCoreApplication::translate("TrackWidget", "days", nullptr));
        hoursLabel->setText(QCoreApplication::translate("TrackWidget", "hours", nullptr));
        minsLabel->setText(QCoreApplication::translate("TrackWidget", "mins", nullptr));
        secsLabel->setText(QCoreApplication::translate("TrackWidget", "secs", nullptr));
#if QT_CONFIG(tooltip)
        startCheck->setToolTip(QCoreApplication::translate("TrackWidget", "Use track pts. after this time. ", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        startCheck->setWhatsThis(QCoreApplication::translate("TrackWidget", "Use only track points after this timestamp.\n"
"\n"
"This option is used along with the stop to discard trackpoints that were recorded outside of a specific period of time. This option specifies the beginning of the time period. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        startCheck->setText(QCoreApplication::translate("TrackWidget", "Start", nullptr));
#if QT_CONFIG(tooltip)
        startEdit->setToolTip(QCoreApplication::translate("TrackWidget", "Use track pts. after this time. ", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        stopCheck->setToolTip(QCoreApplication::translate("TrackWidget", "Use track pts before this time. ", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        stopCheck->setWhatsThis(QCoreApplication::translate("TrackWidget", " Use only track points before this timestamp.\n"
"\n"
"This option is used in conjunction with the start option to discard all trackpoints outside of a given period of time. This option defines the end of the time period. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        stopCheck->setText(QCoreApplication::translate("TrackWidget", "Stop", nullptr));
#if QT_CONFIG(tooltip)
        stopEdit->setToolTip(QCoreApplication::translate("TrackWidget", "Use track pts before this time. ", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        packCheck->setToolTip(QCoreApplication::translate("TrackWidget", "Pack all tracks into one. ", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        packCheck->setWhatsThis(QCoreApplication::translate("TrackWidget", "Pack all tracks into one.\n"
"\n"
"This option causes all tracks to be appended to one another to form a single track. This option does not work if any two tracks overlap in time; in that case, consider using the merge option.\n"
"\n"
"This option is most useful for rejoining tracks that might have been interrupted by an equipment malfunction or an overnight stop. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        packCheck->setText(QCoreApplication::translate("TrackWidget", "Pack", nullptr));
#if QT_CONFIG(tooltip)
        mergeCheck->setToolTip(QCoreApplication::translate("TrackWidget", "Merge multiple tracks for the same way.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mergeCheck->setWhatsThis(QCoreApplication::translate("TrackWidget", " Merge multiple tracks for the same way.\n"
"\n"
"This option puts all track points from all tracks into a single track and sorts them by time stamp. Points with identical time stamps will be dropped. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        mergeCheck->setText(QCoreApplication::translate("TrackWidget", "Merge", nullptr));
        splitDateCheck->setText(QCoreApplication::translate("TrackWidget", "Split by Date", nullptr));
        splitTimeCheck->setText(QCoreApplication::translate("TrackWidget", "Split by Time", nullptr));
#if QT_CONFIG(tooltip)
        splitTimeSpin->setToolTip(QCoreApplication::translate("TrackWidget", "If nonzero, the track will be split if the time between two points is greater than this parameter.   If zero, the track will be split by date. ", nullptr));
#endif // QT_CONFIG(tooltip)
        splitTimeCombo->setItemText(0, QCoreApplication::translate("TrackWidget", "mins", nullptr));
        splitTimeCombo->setItemText(1, QCoreApplication::translate("TrackWidget", "hrs", nullptr));
        splitTimeCombo->setItemText(2, QCoreApplication::translate("TrackWidget", "days", nullptr));

        splitDistanceCheck->setText(QCoreApplication::translate("TrackWidget", "Split by Dist.", nullptr));
#if QT_CONFIG(tooltip)
        splitDistSpin->setToolTip(QCoreApplication::translate("TrackWidget", "If nonzero, the input track will be split into several tracks if the distance between successive track points is greater than the distance given as a parameter. ", nullptr));
#endif // QT_CONFIG(tooltip)
        splitDistCombo->setItemText(0, QCoreApplication::translate("TrackWidget", "ft", nullptr));
        splitDistCombo->setItemText(1, QCoreApplication::translate("TrackWidget", "m", nullptr));
        splitDistCombo->setItemText(2, QCoreApplication::translate("TrackWidget", "km", nullptr));
        splitDistCombo->setItemText(3, QCoreApplication::translate("TrackWidget", "mi", nullptr));

        GPSFixesCheck->setText(QCoreApplication::translate("TrackWidget", "GPS Fixes", nullptr));
        GPSFixesCombo->setItemText(0, QCoreApplication::translate("TrackWidget", "none", nullptr));
        GPSFixesCombo->setItemText(1, QCoreApplication::translate("TrackWidget", "pps", nullptr));
        GPSFixesCombo->setItemText(2, QCoreApplication::translate("TrackWidget", "dgps", nullptr));
        GPSFixesCombo->setItemText(3, QCoreApplication::translate("TrackWidget", "3d", nullptr));
        GPSFixesCombo->setItemText(4, QCoreApplication::translate("TrackWidget", "2d", nullptr));

#if QT_CONFIG(tooltip)
        speedCheck->setToolTip(QCoreApplication::translate("TrackWidget", "Synthesize speed. ", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        speedCheck->setWhatsThis(QCoreApplication::translate("TrackWidget", "Synthesize speed.\n"
"\n"
"This option computes a value for the GPS speed at each trackpoint. This is most useful with trackpoints from formats that don't support speed information or for trackpoints synthesized by the interpolate filter. The speed at each trackpoint is the average speed from the previous trackpoint (distance divided by time). The first trackpoint in each track is assigned a speed of \"unknown.\" ", nullptr));
#endif // QT_CONFIG(whatsthis)
        speedCheck->setText(QCoreApplication::translate("TrackWidget", "Speed", nullptr));
#if QT_CONFIG(tooltip)
        courseCheck->setToolTip(QCoreApplication::translate("TrackWidget", "Synthesize course.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        courseCheck->setWhatsThis(QCoreApplication::translate("TrackWidget", "Synthesize course.\n"
"\n"
"This option computes (or recomputes) a value for the GPS heading at each trackpoint. This is most useful with trackpoints from formats that don't support heading information or for trackpoints synthesized by the interpolate filter. The heading at each trackpoint is simply the course from the previous trackpoint in the track. The first trackpoint in each track is arbitrarily assigned a heading of 0 degrees. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        courseCheck->setText(QCoreApplication::translate("TrackWidget", "Course", nullptr));
#if QT_CONFIG(tooltip)
        localTime->setToolTip(QCoreApplication::translate("TrackWidget", "If checked, times specified here are based on this computer's current time zone.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        localTime->setWhatsThis(QCoreApplication::translate("TrackWidget", "If checked, times specified here are based on this computer's current time zone.", nullptr));
#endif // QT_CONFIG(whatsthis)
        localTime->setText(QCoreApplication::translate("TrackWidget", "Local Time", nullptr));
#if QT_CONFIG(tooltip)
        utc->setToolTip(QCoreApplication::translate("TrackWidget", "If checked, times specified here are UTC.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        utc->setWhatsThis(QCoreApplication::translate("TrackWidget", "If checked, times specified here are UTC.", nullptr));
#endif // QT_CONFIG(whatsthis)
        utc->setText(QCoreApplication::translate("TrackWidget", "UTC", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TrackWidget: public Ui_TrackWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TRACKUI_H
