/********************************************************************************
** Form generated from reading UI file 'rttrkui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RTTRKUI_H
#define UI_RTTRKUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_RtTrkWidget
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QCheckBox *simplifyCheck;
    QSpacerItem *horizontalSpacer;
    QLabel *limitToLabel;
    QSpinBox *limitToSpin;
    QLabel *pointLabel;
    QSpacerItem *horizontalSpacer_2;
    QCheckBox *reverseCheck;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *RtTrkWidget)
    {
        if (RtTrkWidget->objectName().isEmpty())
            RtTrkWidget->setObjectName("RtTrkWidget");
        RtTrkWidget->resize(305, 114);
        verticalLayout = new QVBoxLayout(RtTrkWidget);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(RtTrkWidget);
        label->setObjectName("label");
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        label->setFont(font);

        verticalLayout->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        simplifyCheck = new QCheckBox(RtTrkWidget);
        simplifyCheck->setObjectName("simplifyCheck");

        horizontalLayout->addWidget(simplifyCheck);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        limitToLabel = new QLabel(RtTrkWidget);
        limitToLabel->setObjectName("limitToLabel");

        horizontalLayout->addWidget(limitToLabel);

        limitToSpin = new QSpinBox(RtTrkWidget);
        limitToSpin->setObjectName("limitToSpin");
        limitToSpin->setStepType(QAbstractSpinBox::AdaptiveDecimalStepType);

        horizontalLayout->addWidget(limitToSpin);

        pointLabel = new QLabel(RtTrkWidget);
        pointLabel->setObjectName("pointLabel");

        horizontalLayout->addWidget(pointLabel);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout);

        reverseCheck = new QCheckBox(RtTrkWidget);
        reverseCheck->setObjectName("reverseCheck");

        verticalLayout->addWidget(reverseCheck);

        verticalSpacer = new QSpacerItem(20, 19, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        QWidget::setTabOrder(simplifyCheck, limitToSpin);
        QWidget::setTabOrder(limitToSpin, reverseCheck);

        retranslateUi(RtTrkWidget);

        QMetaObject::connectSlotsByName(RtTrkWidget);
    } // setupUi

    void retranslateUi(QWidget *RtTrkWidget)
    {
        RtTrkWidget->setWindowTitle(QCoreApplication::translate("RtTrkWidget", "Form", nullptr));
#if QT_CONFIG(tooltip)
        RtTrkWidget->setToolTip(QCoreApplication::translate("RtTrkWidget", "Simplify routes and tracks by removing points", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        RtTrkWidget->setWhatsThis(QCoreApplication::translate("RtTrkWidget", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'DejaVu Sans'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The Simplify filter is used to simplify routes and tracks for use with formats that limit the number of points they can contain or just to reduce the complexity of a route. </p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The filter attempts to remove points from each route until the number of points or the error is within the given bounds, while also attempting to preserve the shape of the original route as much as possible. </p>\n"
"<p style=\" margin-top:12px; margin-bottom"
                        ":12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The quality of the results will vary depending on the density of points in the original route and the length of the original route. </p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        label->setText(QCoreApplication::translate("RtTrkWidget", "Routes & Tracks", nullptr));
#if QT_CONFIG(tooltip)
        simplifyCheck->setToolTip(QCoreApplication::translate("RtTrkWidget", "Simplify route by removing points", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        simplifyCheck->setWhatsThis(QCoreApplication::translate("RtTrkWidget", " The Simplify filter is used to simplify routes and tracks for use with formats that limit the number of points they can contain or just to reduce the complexity of a route.\n"
"\n"
"The filter attempts to remove points from each route until the number of points or the error is within the given bounds, while also attempting to preserve the shape of the original route as much as possible.\n"
"\n"
"The quality of the results will vary depending on the density of points in the original route and the length of the original route. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        simplifyCheck->setText(QCoreApplication::translate("RtTrkWidget", "Simplify", nullptr));
        limitToLabel->setText(QCoreApplication::translate("RtTrkWidget", "Limit To", nullptr));
#if QT_CONFIG(tooltip)
        limitToSpin->setToolTip(QCoreApplication::translate("RtTrkWidget", "Maximum number points in track or route. ", nullptr));
#endif // QT_CONFIG(tooltip)
        pointLabel->setText(QCoreApplication::translate("RtTrkWidget", "Points", nullptr));
#if QT_CONFIG(tooltip)
        reverseCheck->setToolTip(QCoreApplication::translate("RtTrkWidget", "Reverse tracks and routes", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        reverseCheck->setWhatsThis(QCoreApplication::translate("RtTrkWidget", "The reversal is performed in the laziest way possible. Timestamps are kept with the original waypoints so the resulting track or route will have the interesting characteristic that time runs backwards. This tends to make Magellan Mapsend, in particular, do a weird thing and place each waypoint on a separate day. \n"
"Additionally, if you're using this to reverse a route that navigates, say, an exit ramp or a one way street, you will be in for unpleasant ride. application cares about timestamps ", nullptr));
#endif // QT_CONFIG(whatsthis)
        reverseCheck->setText(QCoreApplication::translate("RtTrkWidget", "Reverse", nullptr));
    } // retranslateUi

};

namespace Ui {
    class RtTrkWidget: public Ui_RtTrkWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RTTRKUI_H
