/********************************************************************************
** Form generated from reading UI file 'miscfltui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MISCFLTUI_H
#define UI_MISCFLTUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MiscFltWidget
{
public:
    QGridLayout *gridLayout_2;
    QGridLayout *gridLayout;
    QCheckBox *sortRteCheck;
    QHBoxLayout *horizontalLayout;
    QCheckBox *transformCheck;
    QComboBox *transformCombo;
    QCheckBox *deleteCheck;
    QSpacerItem *horizontalSpacer;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *nukeRoutes;
    QCheckBox *nukeTracks;
    QCheckBox *nukeWaypoints;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_2;
    QComboBox *sortWptBy;
    QSpacerItem *horizontalSpacer_2;
    QCheckBox *sortWptCheck;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_3;
    QComboBox *sortRteBy;
    QSpacerItem *horizontalSpacer_3;
    QCheckBox *swapCheck;
    QLabel *label;
    QCheckBox *sortTrkCheck;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_4;
    QComboBox *sortTrkBy;
    QSpacerItem *horizontalSpacer_4;

    void setupUi(QWidget *MiscFltWidget)
    {
        if (MiscFltWidget->objectName().isEmpty())
            MiscFltWidget->setObjectName("MiscFltWidget");
        MiscFltWidget->resize(310, 250);
        gridLayout_2 = new QGridLayout(MiscFltWidget);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        sortRteCheck = new QCheckBox(MiscFltWidget);
        sortRteCheck->setObjectName("sortRteCheck");

        gridLayout->addWidget(sortRteCheck, 5, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        transformCheck = new QCheckBox(MiscFltWidget);
        transformCheck->setObjectName("transformCheck");

        horizontalLayout->addWidget(transformCheck);

        transformCombo = new QComboBox(MiscFltWidget);
        transformCombo->setObjectName("transformCombo");

        horizontalLayout->addWidget(transformCombo);

        deleteCheck = new QCheckBox(MiscFltWidget);
        deleteCheck->setObjectName("deleteCheck");

        horizontalLayout->addWidget(deleteCheck);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        gridLayout->addLayout(horizontalLayout, 2, 0, 1, 2);

        groupBox = new QGroupBox(MiscFltWidget);
        groupBox->setObjectName("groupBox");
        horizontalLayout_2 = new QHBoxLayout(groupBox);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(-1, 4, -1, 4);
        nukeRoutes = new QCheckBox(groupBox);
        nukeRoutes->setObjectName("nukeRoutes");

        horizontalLayout_2->addWidget(nukeRoutes);

        nukeTracks = new QCheckBox(groupBox);
        nukeTracks->setObjectName("nukeTracks");

        horizontalLayout_2->addWidget(nukeTracks);

        nukeWaypoints = new QCheckBox(groupBox);
        nukeWaypoints->setObjectName("nukeWaypoints");

        horizontalLayout_2->addWidget(nukeWaypoints);


        gridLayout->addWidget(groupBox, 1, 0, 1, 2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_2 = new QLabel(MiscFltWidget);
        label_2->setObjectName("label_2");

        horizontalLayout_3->addWidget(label_2);

        sortWptBy = new QComboBox(MiscFltWidget);
        sortWptBy->addItem(QString());
        sortWptBy->addItem(QString());
        sortWptBy->addItem(QString());
        sortWptBy->addItem(QString());
        sortWptBy->setObjectName("sortWptBy");

        horizontalLayout_3->addWidget(sortWptBy);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);


        gridLayout->addLayout(horizontalLayout_3, 4, 1, 1, 1);

        sortWptCheck = new QCheckBox(MiscFltWidget);
        sortWptCheck->setObjectName("sortWptCheck");

        gridLayout->addWidget(sortWptCheck, 4, 0, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_3 = new QLabel(MiscFltWidget);
        label_3->setObjectName("label_3");

        horizontalLayout_4->addWidget(label_3);

        sortRteBy = new QComboBox(MiscFltWidget);
        sortRteBy->addItem(QString());
        sortRteBy->addItem(QString());
        sortRteBy->addItem(QString());
        sortRteBy->setObjectName("sortRteBy");

        horizontalLayout_4->addWidget(sortRteBy);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_3);


        gridLayout->addLayout(horizontalLayout_4, 5, 1, 1, 1);

        swapCheck = new QCheckBox(MiscFltWidget);
        swapCheck->setObjectName("swapCheck");

        gridLayout->addWidget(swapCheck, 3, 0, 1, 2);

        label = new QLabel(MiscFltWidget);
        label->setObjectName("label");
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        label->setFont(font);

        gridLayout->addWidget(label, 0, 0, 1, 1);

        sortTrkCheck = new QCheckBox(MiscFltWidget);
        sortTrkCheck->setObjectName("sortTrkCheck");

        gridLayout->addWidget(sortTrkCheck, 6, 0, 1, 1);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label_4 = new QLabel(MiscFltWidget);
        label_4->setObjectName("label_4");

        horizontalLayout_5->addWidget(label_4);

        sortTrkBy = new QComboBox(MiscFltWidget);
        sortTrkBy->addItem(QString());
        sortTrkBy->addItem(QString());
        sortTrkBy->addItem(QString());
        sortTrkBy->setObjectName("sortTrkBy");

        horizontalLayout_5->addWidget(sortTrkBy);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_4);


        gridLayout->addLayout(horizontalLayout_5, 6, 1, 1, 1);


        gridLayout_2->addLayout(gridLayout, 0, 0, 1, 1);

        QWidget::setTabOrder(nukeRoutes, nukeTracks);
        QWidget::setTabOrder(nukeTracks, nukeWaypoints);
        QWidget::setTabOrder(nukeWaypoints, transformCheck);
        QWidget::setTabOrder(transformCheck, transformCombo);
        QWidget::setTabOrder(transformCombo, deleteCheck);
        QWidget::setTabOrder(deleteCheck, swapCheck);
        QWidget::setTabOrder(swapCheck, sortWptCheck);
        QWidget::setTabOrder(sortWptCheck, sortWptBy);
        QWidget::setTabOrder(sortWptBy, sortRteCheck);
        QWidget::setTabOrder(sortRteCheck, sortRteBy);
        QWidget::setTabOrder(sortRteBy, sortTrkCheck);
        QWidget::setTabOrder(sortTrkCheck, sortTrkBy);

        retranslateUi(MiscFltWidget);

        QMetaObject::connectSlotsByName(MiscFltWidget);
    } // setupUi

    void retranslateUi(QWidget *MiscFltWidget)
    {
        MiscFltWidget->setWindowTitle(QCoreApplication::translate("MiscFltWidget", "Form", nullptr));
        sortRteCheck->setText(QCoreApplication::translate("MiscFltWidget", "Sort Routes", nullptr));
#if QT_CONFIG(tooltip)
        transformCheck->setToolTip(QCoreApplication::translate("MiscFltWidget", "Convert routes, waypoints and tracks to different types.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        transformCheck->setWhatsThis(QCoreApplication::translate("MiscFltWidget", "This filter can be used to convert GPS data between different data types.\n"
"\n"
"Some GPS data formats support only some subset of waypoints, tracks, and routes. The transform filter allows you to convert between these types. For example, it can be used to convert a pile of waypoints (such as those from a CSV file) into a track or vice versa. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        transformCheck->setText(QCoreApplication::translate("MiscFltWidget", "Transform", nullptr));
#if QT_CONFIG(tooltip)
        transformCombo->setToolTip(QCoreApplication::translate("MiscFltWidget", "Type of transformation. ", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        deleteCheck->setToolTip(QCoreApplication::translate("MiscFltWidget", "Delete original data after transform to prevent duplicated data. ", nullptr));
#endif // QT_CONFIG(tooltip)
        deleteCheck->setText(QCoreApplication::translate("MiscFltWidget", "Delete", nullptr));
        groupBox->setTitle(QCoreApplication::translate("MiscFltWidget", "Nuke (Remove) Data Types", nullptr));
        nukeRoutes->setText(QCoreApplication::translate("MiscFltWidget", "Routes", nullptr));
        nukeTracks->setText(QCoreApplication::translate("MiscFltWidget", "Tracks", nullptr));
        nukeWaypoints->setText(QCoreApplication::translate("MiscFltWidget", "Waypoints", nullptr));
        label_2->setText(QCoreApplication::translate("MiscFltWidget", "By", nullptr));
        sortWptBy->setItemText(0, QCoreApplication::translate("MiscFltWidget", "Description", nullptr));
        sortWptBy->setItemText(1, QCoreApplication::translate("MiscFltWidget", "GCID", nullptr));
        sortWptBy->setItemText(2, QCoreApplication::translate("MiscFltWidget", "Name", nullptr));
        sortWptBy->setItemText(3, QCoreApplication::translate("MiscFltWidget", "Time", nullptr));

        sortWptCheck->setText(QCoreApplication::translate("MiscFltWidget", "Sort Waypoints", nullptr));
        label_3->setText(QCoreApplication::translate("MiscFltWidget", "By", nullptr));
        sortRteBy->setItemText(0, QCoreApplication::translate("MiscFltWidget", "Description", nullptr));
        sortRteBy->setItemText(1, QCoreApplication::translate("MiscFltWidget", "Name", nullptr));
        sortRteBy->setItemText(2, QCoreApplication::translate("MiscFltWidget", "Number", nullptr));

#if QT_CONFIG(tooltip)
        swapCheck->setToolTip(QCoreApplication::translate("MiscFltWidget", "Swap Longitude and Latitudes for badly formatted data formats.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        swapCheck->setWhatsThis(QCoreApplication::translate("MiscFltWidget", "Simple filter to swap the coordinate values (latitude and longitude) of all points. This can be helpful for wrong defined/coded data. Or if you think, you can use one of our xcsv formats, but latitude and longitude are in opposite order. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        swapCheck->setText(QCoreApplication::translate("MiscFltWidget", "Swap Coordinates", nullptr));
        label->setText(QCoreApplication::translate("MiscFltWidget", "Misc. Filters", nullptr));
        sortTrkCheck->setText(QCoreApplication::translate("MiscFltWidget", "Sort Tracks", nullptr));
        label_4->setText(QCoreApplication::translate("MiscFltWidget", "By", nullptr));
        sortTrkBy->setItemText(0, QCoreApplication::translate("MiscFltWidget", "Description", nullptr));
        sortTrkBy->setItemText(1, QCoreApplication::translate("MiscFltWidget", "Name", nullptr));
        sortTrkBy->setItemText(2, QCoreApplication::translate("MiscFltWidget", "Number", nullptr));

    } // retranslateUi

};

namespace Ui {
    class MiscFltWidget: public Ui_MiscFltWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MISCFLTUI_H
