/********************************************************************************
** Form generated from reading UI file 'donate.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DONATE_H
#define UI_DONATE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Donate
{
public:
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    QLabel *textLine1;
    QSpacerItem *verticalSpacer_2;
    QLabel *textLine2;
    QCheckBox *neverAgain;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *dismissButton;
    QPushButton *contributeButton;

    void setupUi(QDialog *Donate)
    {
        if (Donate->objectName().isEmpty())
            Donate->setObjectName("Donate");
        Donate->resize(350, 350);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Donate->sizePolicy().hasHeightForWidth());
        Donate->setSizePolicy(sizePolicy);
        Donate->setMinimumSize(QSize(0, 0));
        Donate->setMaximumSize(QSize(400, 400));
        layoutWidget = new QWidget(Donate);
        layoutWidget->setObjectName("layoutWidget");
        layoutWidget->setGeometry(QRect(10, 13, 321, 321));
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        textLine1 = new QLabel(layoutWidget);
        textLine1->setObjectName("textLine1");
        QFont font;
        font.setPointSize(12);
        textLine1->setFont(font);
        textLine1->setWordWrap(true);
        textLine1->setOpenExternalLinks(true);

        verticalLayout->addWidget(textLine1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);

        textLine2 = new QLabel(layoutWidget);
        textLine2->setObjectName("textLine2");
        textLine2->setWordWrap(true);

        verticalLayout->addWidget(textLine2);

        neverAgain = new QCheckBox(layoutWidget);
        neverAgain->setObjectName("neverAgain");

        verticalLayout->addWidget(neverAgain);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        dismissButton = new QPushButton(layoutWidget);
        dismissButton->setObjectName("dismissButton");

        horizontalLayout->addWidget(dismissButton);

        contributeButton = new QPushButton(layoutWidget);
        contributeButton->setObjectName("contributeButton");
        contributeButton->setAutoDefault(false);

        horizontalLayout->addWidget(contributeButton);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(Donate);
        QObject::connect(dismissButton, &QPushButton::clicked, Donate, qOverload<>(&QDialog::close));

        contributeButton->setDefault(true);


        QMetaObject::connectSlotsByName(Donate);
    } // setupUi

    void retranslateUi(QDialog *Donate)
    {
        Donate->setWindowTitle(QCoreApplication::translate("Donate", "Support GPSBabel", nullptr));
        textLine1->setText(QCoreApplication::translate("Donate", "<html><head/><body><p>GPSBabel is free software built and supported by volunteers. It consumes vast amounts of time to create and support the software as well as money for mapping programs, GPS receivers, and development fixtures. Please see how you can <a href=\"https://www.gpsbabel.org/contribute.html\"><span style=\" text-decoration: underline; color:#0000ff;\">contribute time or via PayPal (no account needed).</span></a></p></body></html>", nullptr));
        textLine2->setText(QCoreApplication::translate("Donate", "<p>Of course, if you've already contributed  to the project or you just can't help the project, please check the box below to never see this message again.</p>", nullptr));
        neverAgain->setText(QCoreApplication::translate("Donate", "Never show this message again.", nullptr));
        dismissButton->setText(QCoreApplication::translate("Donate", "No, Thanks", nullptr));
        contributeButton->setText(QCoreApplication::translate("Donate", "Contribute", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Donate: public Ui_Donate {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DONATE_H
