/********************************************************************************
** Form generated from reading UI file 'filterui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILTERUI_H
#define UI_FILTERUI_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_FilterDlg
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QListWidget *filterList;
    QGridLayout *gridLayout;
    QFrame *frame;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *helpButton;
    QPushButton *resetButton;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *FilterDlg)
    {
        if (FilterDlg->objectName().isEmpty())
            FilterDlg->setObjectName("FilterDlg");
        FilterDlg->resize(892, 260);
        verticalLayout = new QVBoxLayout(FilterDlg);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        filterList = new QListWidget(FilterDlg);
        filterList->setObjectName("filterList");
        filterList->setMinimumSize(QSize(140, 0));

        horizontalLayout->addWidget(filterList);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        frame = new QFrame(FilterDlg);
        frame->setObjectName("frame");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(100);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy);
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);

        gridLayout->addWidget(frame, 0, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 1, 0, 1, 1);


        horizontalLayout->addLayout(gridLayout);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        helpButton = new QPushButton(FilterDlg);
        helpButton->setObjectName("helpButton");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/help.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        helpButton->setIcon(icon);

        horizontalLayout_2->addWidget(helpButton);

        resetButton = new QPushButton(FilterDlg);
        resetButton->setObjectName("resetButton");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/reload.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        resetButton->setIcon(icon1);

        horizontalLayout_2->addWidget(resetButton);

        buttonBox = new QDialogButtonBox(FilterDlg);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        horizontalLayout_2->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout_2);


        retranslateUi(FilterDlg);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, FilterDlg, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, FilterDlg, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(FilterDlg);
    } // setupUi

    void retranslateUi(QDialog *FilterDlg)
    {
        FilterDlg->setWindowTitle(QCoreApplication::translate("FilterDlg", "Data Filters", nullptr));
#if QT_CONFIG(whatsthis)
        FilterDlg->setWhatsThis(QCoreApplication::translate("FilterDlg", "Data filters process and transform the data between input and output files or devices. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        helpButton->setText(QCoreApplication::translate("FilterDlg", "Help", nullptr));
        resetButton->setText(QCoreApplication::translate("FilterDlg", "Reset", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FilterDlg: public Ui_FilterDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILTERUI_H
