/********************************************************************************
** Form generated from reading UI file 'version_mismatch.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_VERSION_MISMATCH_H
#define UI_VERSION_MISMATCH_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_VersionMismatch
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QLabel *ClVersion;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_3;
    QLabel *GuiVersion;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *neverAgain;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *VersionMismatch)
    {
        if (VersionMismatch->objectName().isEmpty())
            VersionMismatch->setObjectName("VersionMismatch");
        VersionMismatch->resize(346, 144);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(VersionMismatch->sizePolicy().hasHeightForWidth());
        VersionMismatch->setSizePolicy(sizePolicy);
        VersionMismatch->setMinimumSize(QSize(0, 0));
        VersionMismatch->setMaximumSize(QSize(400, 300));
        verticalLayout = new QVBoxLayout(VersionMismatch);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(VersionMismatch);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_2 = new QLabel(VersionMismatch);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        ClVersion = new QLabel(VersionMismatch);
        ClVersion->setObjectName("ClVersion");
#if QT_CONFIG(tooltip)
        ClVersion->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)

        horizontalLayout->addWidget(ClVersion);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_3 = new QLabel(VersionMismatch);
        label_3->setObjectName("label_3");

        horizontalLayout_2->addWidget(label_3);

        GuiVersion = new QLabel(VersionMismatch);
        GuiVersion->setObjectName("GuiVersion");

        horizontalLayout_2->addWidget(GuiVersion);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        neverAgain = new QCheckBox(VersionMismatch);
        neverAgain->setObjectName("neverAgain");

        horizontalLayout_3->addWidget(neverAgain);

        buttonBox = new QDialogButtonBox(VersionMismatch);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Ignore);

        horizontalLayout_3->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout_3);


        retranslateUi(VersionMismatch);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, VersionMismatch, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, VersionMismatch, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(VersionMismatch);
    } // setupUi

    void retranslateUi(QDialog *VersionMismatch)
    {
        VersionMismatch->setWindowTitle(QCoreApplication::translate("VersionMismatch", "GPSBabel Version Mismatch", nullptr));
        label->setText(QCoreApplication::translate("VersionMismatch", "<b>A version mismatch has been detected.</b>", nullptr));
        label_2->setText(QCoreApplication::translate("VersionMismatch", "GPSBabel command line version:", nullptr));
        ClVersion->setText(QString());
        label_3->setText(QCoreApplication::translate("VersionMismatch", "GPSBabel GUI version:", nullptr));
#if QT_CONFIG(tooltip)
        GuiVersion->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        GuiVersion->setText(QString());
        neverAgain->setText(QCoreApplication::translate("VersionMismatch", "Never show this message again.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class VersionMismatch: public Ui_VersionMismatch {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_VERSION_MISMATCH_H
