/********************************************************************************
** Form generated from reading UI file 'wayptsui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WAYPTSUI_H
#define UI_WAYPTSUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_WayPtsWidget
{
public:
    QGridLayout *gridLayout_2;
    QGridLayout *gridLayout;
    QCheckBox *radiusCheck;
    QCheckBox *positionCheck;
    QLabel *latLabel;
    QLineEdit *latText;
    QComboBox *radiusUnitCombo;
    QLabel *label;
    QLineEdit *longText;
    QLineEdit *positionText;
    QLabel *longLabel;
    QCheckBox *locationsCheck;
    QComboBox *positionUnitCombo;
    QCheckBox *duplicatesCheck;
    QCheckBox *shortNamesCheck;
    QLineEdit *radiusText;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *WayPtsWidget)
    {
        if (WayPtsWidget->objectName().isEmpty())
            WayPtsWidget->setObjectName("WayPtsWidget");
        WayPtsWidget->resize(466, 173);
        gridLayout_2 = new QGridLayout(WayPtsWidget);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        radiusCheck = new QCheckBox(WayPtsWidget);
        radiusCheck->setObjectName("radiusCheck");

        gridLayout->addWidget(radiusCheck, 3, 0, 1, 1);

        positionCheck = new QCheckBox(WayPtsWidget);
        positionCheck->setObjectName("positionCheck");

        gridLayout->addWidget(positionCheck, 2, 0, 1, 1);

        latLabel = new QLabel(WayPtsWidget);
        latLabel->setObjectName("latLabel");

        gridLayout->addWidget(latLabel, 3, 3, 1, 1);

        latText = new QLineEdit(WayPtsWidget);
        latText->setObjectName("latText");
        latText->setMinimumSize(QSize(110, 0));

        gridLayout->addWidget(latText, 3, 4, 1, 1);

        radiusUnitCombo = new QComboBox(WayPtsWidget);
        radiusUnitCombo->addItem(QString());
        radiusUnitCombo->addItem(QString());
        radiusUnitCombo->setObjectName("radiusUnitCombo");

        gridLayout->addWidget(radiusUnitCombo, 3, 2, 1, 1);

        label = new QLabel(WayPtsWidget);
        label->setObjectName("label");
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        label->setFont(font);

        gridLayout->addWidget(label, 0, 0, 1, 5);

        longText = new QLineEdit(WayPtsWidget);
        longText->setObjectName("longText");
        longText->setMinimumSize(QSize(110, 0));

        gridLayout->addWidget(longText, 4, 4, 1, 1);

        positionText = new QLineEdit(WayPtsWidget);
        positionText->setObjectName("positionText");
        positionText->setMaximumSize(QSize(80, 16777215));

        gridLayout->addWidget(positionText, 2, 1, 1, 1);

        longLabel = new QLabel(WayPtsWidget);
        longLabel->setObjectName("longLabel");

        gridLayout->addWidget(longLabel, 4, 3, 1, 1);

        locationsCheck = new QCheckBox(WayPtsWidget);
        locationsCheck->setObjectName("locationsCheck");

        gridLayout->addWidget(locationsCheck, 1, 2, 1, 1);

        positionUnitCombo = new QComboBox(WayPtsWidget);
        positionUnitCombo->addItem(QString());
        positionUnitCombo->addItem(QString());
        positionUnitCombo->setObjectName("positionUnitCombo");

        gridLayout->addWidget(positionUnitCombo, 2, 2, 1, 1);

        duplicatesCheck = new QCheckBox(WayPtsWidget);
        duplicatesCheck->setObjectName("duplicatesCheck");

        gridLayout->addWidget(duplicatesCheck, 1, 0, 1, 1);

        shortNamesCheck = new QCheckBox(WayPtsWidget);
        shortNamesCheck->setObjectName("shortNamesCheck");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(shortNamesCheck->sizePolicy().hasHeightForWidth());
        shortNamesCheck->setSizePolicy(sizePolicy);

        gridLayout->addWidget(shortNamesCheck, 1, 1, 1, 1);

        radiusText = new QLineEdit(WayPtsWidget);
        radiusText->setObjectName("radiusText");
        radiusText->setMaximumSize(QSize(80, 16777215));

        gridLayout->addWidget(radiusText, 3, 1, 1, 1);


        gridLayout_2->addLayout(gridLayout, 0, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(80, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 0, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 1, 0, 1, 1);

        QWidget::setTabOrder(duplicatesCheck, shortNamesCheck);
        QWidget::setTabOrder(shortNamesCheck, locationsCheck);
        QWidget::setTabOrder(locationsCheck, positionCheck);
        QWidget::setTabOrder(positionCheck, positionText);
        QWidget::setTabOrder(positionText, positionUnitCombo);
        QWidget::setTabOrder(positionUnitCombo, radiusCheck);
        QWidget::setTabOrder(radiusCheck, radiusText);
        QWidget::setTabOrder(radiusText, radiusUnitCombo);
        QWidget::setTabOrder(radiusUnitCombo, latText);
        QWidget::setTabOrder(latText, longText);

        retranslateUi(WayPtsWidget);

        QMetaObject::connectSlotsByName(WayPtsWidget);
    } // setupUi

    void retranslateUi(QWidget *WayPtsWidget)
    {
        WayPtsWidget->setWindowTitle(QCoreApplication::translate("WayPtsWidget", "Form", nullptr));
#if QT_CONFIG(tooltip)
        radiusCheck->setToolTip(QCoreApplication::translate("WayPtsWidget", "Include points only within radius", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        radiusCheck->setWhatsThis(QCoreApplication::translate("WayPtsWidget", "This filter includes or excludes waypoints based on their proximity to a central point. All waypoints more than the specified distance from the specified point will be removed from the dataset.\n"
"\n"
"By default, all remaining points are sorted so that points closer to the center appear earlier in the output file. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        radiusCheck->setText(QCoreApplication::translate("WayPtsWidget", "Radius", nullptr));
#if QT_CONFIG(tooltip)
        positionCheck->setToolTip(QCoreApplication::translate("WayPtsWidget", "Remove points based on proximity", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        positionCheck->setWhatsThis(QCoreApplication::translate("WayPtsWidget", "Maximum positional distance.\n"
"\n"
"This option specifies the minimum allowable distance between two points. If two points are closer than this distance, only one of them is kept. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        positionCheck->setText(QCoreApplication::translate("WayPtsWidget", "Position", nullptr));
        latLabel->setText(QCoreApplication::translate("WayPtsWidget", "Lat.", nullptr));
#if QT_CONFIG(tooltip)
        latText->setToolTip(QCoreApplication::translate("WayPtsWidget", "Latitude of the central point in decimal degrees.  South latitudes should be expressed as a negative number.", nullptr));
#endif // QT_CONFIG(tooltip)
        radiusUnitCombo->setItemText(0, QCoreApplication::translate("WayPtsWidget", "Miles", nullptr));
        radiusUnitCombo->setItemText(1, QCoreApplication::translate("WayPtsWidget", "km", nullptr));

        label->setText(QCoreApplication::translate("WayPtsWidget", "Waypoints Filters", nullptr));
#if QT_CONFIG(tooltip)
        longText->setToolTip(QCoreApplication::translate("WayPtsWidget", "Longitude of the central point in decimal degrees. West longitudes should be expressed as a negative number.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        positionText->setToolTip(QCoreApplication::translate("WayPtsWidget", "Maximum positional distance.", nullptr));
#endif // QT_CONFIG(tooltip)
        longLabel->setText(QCoreApplication::translate("WayPtsWidget", "Long.", nullptr));
#if QT_CONFIG(tooltip)
        locationsCheck->setToolTip(QCoreApplication::translate("WayPtsWidget", "Suppress duplicate waypoint based on coords. ", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        locationsCheck->setWhatsThis(QCoreApplication::translate("WayPtsWidget", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'DejaVu Sans'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Suppress duplicate waypoint based on coords. </p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">This option causes the duplicate filter to remove any additional waypoint that has the same coordinates (to six decimal degrees) as a waypoint that came before. This option may be used to remove duplicate waypoints if the names are not expected to be the same. It also might be used along with the <span style=\" font-family:'Courier New,courier';\">shortname</span> option to remove dup"
                        "licate waypoints if the names of several unrelated groups of waypoints might be the same. </p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        locationsCheck->setText(QCoreApplication::translate("WayPtsWidget", "Locations", nullptr));
        positionUnitCombo->setItemText(0, QCoreApplication::translate("WayPtsWidget", "Feet", nullptr));
        positionUnitCombo->setItemText(1, QCoreApplication::translate("WayPtsWidget", "Meters", nullptr));

#if QT_CONFIG(tooltip)
        duplicatesCheck->setToolTip(QCoreApplication::translate("WayPtsWidget", "Remove duplicates", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        duplicatesCheck->setWhatsThis(QCoreApplication::translate("WayPtsWidget", "The duplicate filter is designed to remove duplicate points based on their short name (traditionally a waypoint's name on the GPS receiver), and/or their location (to a precision of 6 decimals). This filter supports two options that specify how duplicates will be recognized, shortname and location. Generally, at least one of these options is required. ", nullptr));
#endif // QT_CONFIG(whatsthis)
        duplicatesCheck->setText(QCoreApplication::translate("WayPtsWidget", "Duplicates", nullptr));
#if QT_CONFIG(tooltip)
        shortNamesCheck->setToolTip(QCoreApplication::translate("WayPtsWidget", "Suppress duplicate waypoints based on name.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        shortNamesCheck->setWhatsThis(QCoreApplication::translate("WayPtsWidget", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'DejaVu Sans'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Suppress duplicate waypoints based on name. </p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">This option is the one most often used with the duplicate filter. This option instructs the duplicate filter to remove any waypoints that share a short name with a waypoint that has come before. This option might be used to remove duplicates if you are merging two datasets that were each created in part from a common ancestor dataset. </p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        shortNamesCheck->setText(QCoreApplication::translate("WayPtsWidget", "Short Names", nullptr));
#if QT_CONFIG(tooltip)
        radiusText->setToolTip(QCoreApplication::translate("WayPtsWidget", "Maximum distance from center. ", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class WayPtsWidget: public Ui_WayPtsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_WAYPTSUI_H
