/********************************************************************************
** Form generated from reading UI file 'mainwinui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINUI_H
#define UI_MAINWINUI_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *actionHelp;
    QAction *actionAbout;
    QAction *actionQuit;
    QAction *actionPreferences;
    QAction *actionUpgradeCheck;
    QAction *actionVisit_Website;
    QAction *actionMake_a_Donation;
    QWidget *centralwidget;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_5;
    QRadioButton *inputFileOptBtn;
    QRadioButton *inputDeviceOptBtn;
    QSpacerItem *horizontalSpacer_2a;
    QLabel *label;
    QComboBox *inputFormatCombo;
    QStackedWidget *inputStackedWidget;
    QWidget *inputFilePage;
    QHBoxLayout *horizontalLayout_4;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *inputFileNameBrowseBtn;
    QLineEdit *inputFileNameText;
    QWidget *inputDevicePage;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_2;
    QComboBox *inputDeviceNameCombo;
    QSpacerItem *horizontalSpacer_3a;
    QHBoxLayout *horizontalLayout_6;
    QPushButton *inputOptionsBtn;
    QLineEdit *inputOptionsText;
    QGroupBox *groupBox_2;
    QHBoxLayout *horizontalLayout_7;
    QLabel *wayPtLabel;
    QCheckBox *xlateWayPtsCk;
    QFrame *line;
    QSpacerItem *horizontalSpacer_6;
    QLabel *routeLabel;
    QCheckBox *xlateRoutesCk;
    QFrame *line_2;
    QSpacerItem *horizontalSpacer_7;
    QLabel *trackLabel;
    QCheckBox *xlateTracksCk;
    QSpacerItem *horizontalSpacer_4;
    QLabel *filterStatus;
    QPushButton *xlateFiltersBtn;
    QSpacerItem *horizontalSpacer_8;
    QSpacerItem *horizontalSpacer_5;
    QPushButton *moreOptionButton;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_8;
    QRadioButton *outputFileOptBtn;
    QRadioButton *outputDeviceOptBtn;
    QSpacerItem *horizontalSpacer_2b;
    QLabel *label_4;
    QComboBox *outputFormatCombo;
    QStackedWidget *outputStackedWidget;
    QWidget *outputFilePage;
    QHBoxLayout *horizontalLayout_9;
    QHBoxLayout *horizontalLayout_10;
    QPushButton *outputFileNameBrowseBtn;
    QLineEdit *outputFileNameText;
    QWidget *outputDevicePage;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_11;
    QLabel *label_6;
    QComboBox *outputDeviceNameCombo;
    QSpacerItem *horizontalSpacer_3b;
    QHBoxLayout *horizontalLayout_12;
    QPushButton *outputOptionsBtn;
    QLineEdit *outputOptionsText;
    QPlainTextEdit *outputWindow;
    QDialogButtonBox *buttonBox;
    QMenuBar *menubar;
    QMenu *menuFile;
    QMenu *menuHelp;
    QStatusBar *statusbar;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName("MainWindow");
        MainWindow->resize(675, 582);
        actionHelp = new QAction(MainWindow);
        actionHelp->setObjectName("actionHelp");
        actionAbout = new QAction(MainWindow);
        actionAbout->setObjectName("actionAbout");
        actionQuit = new QAction(MainWindow);
        actionQuit->setObjectName("actionQuit");
        actionPreferences = new QAction(MainWindow);
        actionPreferences->setObjectName("actionPreferences");
        actionUpgradeCheck = new QAction(MainWindow);
        actionUpgradeCheck->setObjectName("actionUpgradeCheck");
        actionVisit_Website = new QAction(MainWindow);
        actionVisit_Website->setObjectName("actionVisit_Website");
        actionMake_a_Donation = new QAction(MainWindow);
        actionMake_a_Donation->setObjectName("actionMake_a_Donation");
        centralwidget = new QWidget(MainWindow);
        centralwidget->setObjectName("centralwidget");
        verticalLayout_5 = new QVBoxLayout(centralwidget);
        verticalLayout_5->setObjectName("verticalLayout_5");
        groupBox = new QGroupBox(centralwidget);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(-1, 4, -1, 4);
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        inputFileOptBtn = new QRadioButton(groupBox);
        inputFileOptBtn->setObjectName("inputFileOptBtn");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(inputFileOptBtn->sizePolicy().hasHeightForWidth());
        inputFileOptBtn->setSizePolicy(sizePolicy);

        horizontalLayout_5->addWidget(inputFileOptBtn);

        inputDeviceOptBtn = new QRadioButton(groupBox);
        inputDeviceOptBtn->setObjectName("inputDeviceOptBtn");
        sizePolicy.setHeightForWidth(inputDeviceOptBtn->sizePolicy().hasHeightForWidth());
        inputDeviceOptBtn->setSizePolicy(sizePolicy);

        horizontalLayout_5->addWidget(inputDeviceOptBtn);

        horizontalSpacer_2a = new QSpacerItem(18, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_2a);

        label = new QLabel(groupBox);
        label->setObjectName("label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);

        horizontalLayout_5->addWidget(label);

        inputFormatCombo = new QComboBox(groupBox);
        inputFormatCombo->setObjectName("inputFormatCombo");

        horizontalLayout_5->addWidget(inputFormatCombo);


        verticalLayout->addLayout(horizontalLayout_5);

        inputStackedWidget = new QStackedWidget(groupBox);
        inputStackedWidget->setObjectName("inputStackedWidget");
        inputStackedWidget->setLineWidth(0);
        inputFilePage = new QWidget();
        inputFilePage->setObjectName("inputFilePage");
        horizontalLayout_4 = new QHBoxLayout(inputFilePage);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        inputFileNameBrowseBtn = new QPushButton(inputFilePage);
        inputFileNameBrowseBtn->setObjectName("inputFileNameBrowseBtn");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/open.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        inputFileNameBrowseBtn->setIcon(icon);

        horizontalLayout_2->addWidget(inputFileNameBrowseBtn);

        inputFileNameText = new QLineEdit(inputFilePage);
        inputFileNameText->setObjectName("inputFileNameText");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(10);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(inputFileNameText->sizePolicy().hasHeightForWidth());
        inputFileNameText->setSizePolicy(sizePolicy2);

        horizontalLayout_2->addWidget(inputFileNameText);


        horizontalLayout_4->addLayout(horizontalLayout_2);

        inputStackedWidget->addWidget(inputFilePage);
        inputDevicePage = new QWidget();
        inputDevicePage->setObjectName("inputDevicePage");
        verticalLayout_2 = new QVBoxLayout(inputDevicePage);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_2 = new QLabel(inputDevicePage);
        label_2->setObjectName("label_2");

        horizontalLayout_3->addWidget(label_2);

        inputDeviceNameCombo = new QComboBox(inputDevicePage);
        inputDeviceNameCombo->setObjectName("inputDeviceNameCombo");
        inputDeviceNameCombo->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        horizontalLayout_3->addWidget(inputDeviceNameCombo);

        horizontalSpacer_3a = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3a);


        verticalLayout_2->addLayout(horizontalLayout_3);

        inputStackedWidget->addWidget(inputDevicePage);

        verticalLayout->addWidget(inputStackedWidget);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        inputOptionsBtn = new QPushButton(groupBox);
        inputOptionsBtn->setObjectName("inputOptionsBtn");

        horizontalLayout_6->addWidget(inputOptionsBtn);

        inputOptionsText = new QLineEdit(groupBox);
        inputOptionsText->setObjectName("inputOptionsText");
        inputOptionsText->setEnabled(true);
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(inputOptionsText->sizePolicy().hasHeightForWidth());
        inputOptionsText->setSizePolicy(sizePolicy3);

        horizontalLayout_6->addWidget(inputOptionsText);


        verticalLayout->addLayout(horizontalLayout_6);


        verticalLayout_5->addWidget(groupBox);

        groupBox_2 = new QGroupBox(centralwidget);
        groupBox_2->setObjectName("groupBox_2");
        horizontalLayout_7 = new QHBoxLayout(groupBox_2);
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        horizontalLayout_7->setContentsMargins(4, 4, 4, 4);
        wayPtLabel = new QLabel(groupBox_2);
        wayPtLabel->setObjectName("wayPtLabel");
        wayPtLabel->setPixmap(QPixmap(QString::fromUtf8(":/images/lights.png")));
        wayPtLabel->setScaledContents(false);

        horizontalLayout_7->addWidget(wayPtLabel);

        xlateWayPtsCk = new QCheckBox(groupBox_2);
        xlateWayPtsCk->setObjectName("xlateWayPtsCk");

        horizontalLayout_7->addWidget(xlateWayPtsCk);

        line = new QFrame(groupBox_2);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::VLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout_7->addWidget(line);

        horizontalSpacer_6 = new QSpacerItem(15, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_6);

        routeLabel = new QLabel(groupBox_2);
        routeLabel->setObjectName("routeLabel");
        routeLabel->setPixmap(QPixmap(QString::fromUtf8(":/images/lights.png")));

        horizontalLayout_7->addWidget(routeLabel);

        xlateRoutesCk = new QCheckBox(groupBox_2);
        xlateRoutesCk->setObjectName("xlateRoutesCk");

        horizontalLayout_7->addWidget(xlateRoutesCk);

        line_2 = new QFrame(groupBox_2);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::VLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout_7->addWidget(line_2);

        horizontalSpacer_7 = new QSpacerItem(15, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_7);

        trackLabel = new QLabel(groupBox_2);
        trackLabel->setObjectName("trackLabel");
        trackLabel->setPixmap(QPixmap(QString::fromUtf8(":/images/lights.png")));

        horizontalLayout_7->addWidget(trackLabel);

        xlateTracksCk = new QCheckBox(groupBox_2);
        xlateTracksCk->setObjectName("xlateTracksCk");

        horizontalLayout_7->addWidget(xlateTracksCk);

        horizontalSpacer_4 = new QSpacerItem(18, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_4);

        filterStatus = new QLabel(groupBox_2);
        filterStatus->setObjectName("filterStatus");
        filterStatus->setPixmap(QPixmap(QString::fromUtf8(":/images/ok20.png")));
        filterStatus->setScaledContents(false);

        horizontalLayout_7->addWidget(filterStatus);

        xlateFiltersBtn = new QPushButton(groupBox_2);
        xlateFiltersBtn->setObjectName("xlateFiltersBtn");

        horizontalLayout_7->addWidget(xlateFiltersBtn);

        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_8);

        horizontalSpacer_5 = new QSpacerItem(244, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_5);

        moreOptionButton = new QPushButton(groupBox_2);
        moreOptionButton->setObjectName("moreOptionButton");

        horizontalLayout_7->addWidget(moreOptionButton);


        verticalLayout_5->addWidget(groupBox_2);

        groupBox_3 = new QGroupBox(centralwidget);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_3 = new QVBoxLayout(groupBox_3);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(-1, 4, -1, 4);
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        outputFileOptBtn = new QRadioButton(groupBox_3);
        outputFileOptBtn->setObjectName("outputFileOptBtn");
        sizePolicy.setHeightForWidth(outputFileOptBtn->sizePolicy().hasHeightForWidth());
        outputFileOptBtn->setSizePolicy(sizePolicy);

        horizontalLayout_8->addWidget(outputFileOptBtn);

        outputDeviceOptBtn = new QRadioButton(groupBox_3);
        outputDeviceOptBtn->setObjectName("outputDeviceOptBtn");
        sizePolicy.setHeightForWidth(outputDeviceOptBtn->sizePolicy().hasHeightForWidth());
        outputDeviceOptBtn->setSizePolicy(sizePolicy);

        horizontalLayout_8->addWidget(outputDeviceOptBtn);

        horizontalSpacer_2b = new QSpacerItem(18, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_2b);

        label_4 = new QLabel(groupBox_3);
        label_4->setObjectName("label_4");
        sizePolicy1.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy1);

        horizontalLayout_8->addWidget(label_4);

        outputFormatCombo = new QComboBox(groupBox_3);
        outputFormatCombo->setObjectName("outputFormatCombo");

        horizontalLayout_8->addWidget(outputFormatCombo);


        verticalLayout_3->addLayout(horizontalLayout_8);

        outputStackedWidget = new QStackedWidget(groupBox_3);
        outputStackedWidget->setObjectName("outputStackedWidget");
        outputFilePage = new QWidget();
        outputFilePage->setObjectName("outputFilePage");
        horizontalLayout_9 = new QHBoxLayout(outputFilePage);
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        horizontalLayout_9->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        outputFileNameBrowseBtn = new QPushButton(outputFilePage);
        outputFileNameBrowseBtn->setObjectName("outputFileNameBrowseBtn");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/save.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        outputFileNameBrowseBtn->setIcon(icon1);

        horizontalLayout_10->addWidget(outputFileNameBrowseBtn);

        outputFileNameText = new QLineEdit(outputFilePage);
        outputFileNameText->setObjectName("outputFileNameText");
        sizePolicy2.setHeightForWidth(outputFileNameText->sizePolicy().hasHeightForWidth());
        outputFileNameText->setSizePolicy(sizePolicy2);

        horizontalLayout_10->addWidget(outputFileNameText);


        horizontalLayout_9->addLayout(horizontalLayout_10);

        outputStackedWidget->addWidget(outputFilePage);
        outputDevicePage = new QWidget();
        outputDevicePage->setObjectName("outputDevicePage");
        verticalLayout_4 = new QVBoxLayout(outputDevicePage);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        label_6 = new QLabel(outputDevicePage);
        label_6->setObjectName("label_6");

        horizontalLayout_11->addWidget(label_6);

        outputDeviceNameCombo = new QComboBox(outputDevicePage);
        outputDeviceNameCombo->setObjectName("outputDeviceNameCombo");
        outputDeviceNameCombo->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        horizontalLayout_11->addWidget(outputDeviceNameCombo);

        horizontalSpacer_3b = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_3b);


        verticalLayout_4->addLayout(horizontalLayout_11);

        outputStackedWidget->addWidget(outputDevicePage);

        verticalLayout_3->addWidget(outputStackedWidget);

        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName("horizontalLayout_12");
        outputOptionsBtn = new QPushButton(groupBox_3);
        outputOptionsBtn->setObjectName("outputOptionsBtn");

        horizontalLayout_12->addWidget(outputOptionsBtn);

        outputOptionsText = new QLineEdit(groupBox_3);
        outputOptionsText->setObjectName("outputOptionsText");

        horizontalLayout_12->addWidget(outputOptionsText);


        verticalLayout_3->addLayout(horizontalLayout_12);


        verticalLayout_5->addWidget(groupBox_3);

        outputWindow = new QPlainTextEdit(centralwidget);
        outputWindow->setObjectName("outputWindow");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(10);
        sizePolicy4.setVerticalStretch(10);
        sizePolicy4.setHeightForWidth(outputWindow->sizePolicy().hasHeightForWidth());
        outputWindow->setSizePolicy(sizePolicy4);

        verticalLayout_5->addWidget(outputWindow);

        buttonBox = new QDialogButtonBox(centralwidget);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Close|QDialogButtonBox::Help|QDialogButtonBox::Ok);

        verticalLayout_5->addWidget(buttonBox);

        MainWindow->setCentralWidget(centralwidget);
        menubar = new QMenuBar(MainWindow);
        menubar->setObjectName("menubar");
        menubar->setGeometry(QRect(0, 0, 675, 26));
        menuFile = new QMenu(menubar);
        menuFile->setObjectName("menuFile");
        menuHelp = new QMenu(menubar);
        menuHelp->setObjectName("menuHelp");
        MainWindow->setMenuBar(menubar);
        statusbar = new QStatusBar(MainWindow);
        statusbar->setObjectName("statusbar");
        MainWindow->setStatusBar(statusbar);

        menubar->addAction(menuFile->menuAction());
        menubar->addAction(menuHelp->menuAction());
        menuFile->addAction(actionPreferences);
        menuFile->addAction(actionQuit);
        menuHelp->addAction(actionHelp);
        menuHelp->addSeparator();
        menuHelp->addAction(actionAbout);
        menuHelp->addAction(actionUpgradeCheck);
        menuHelp->addSeparator();
        menuHelp->addAction(actionVisit_Website);
        menuHelp->addAction(actionMake_a_Donation);

        retranslateUi(MainWindow);

        inputStackedWidget->setCurrentIndex(1);
        outputStackedWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(QCoreApplication::translate("MainWindow", "GPSBabel", nullptr));
        actionHelp->setText(QCoreApplication::translate("MainWindow", "GPSBabel Help", nullptr));
        actionAbout->setText(QCoreApplication::translate("MainWindow", "About GPSBabel", nullptr));
        actionQuit->setText(QCoreApplication::translate("MainWindow", "Quit", nullptr));
        actionPreferences->setText(QCoreApplication::translate("MainWindow", "Preferences...", nullptr));
        actionUpgradeCheck->setText(QCoreApplication::translate("MainWindow", "Check for Upgrade", nullptr));
        actionVisit_Website->setText(QCoreApplication::translate("MainWindow", "Visit Website...", nullptr));
        actionMake_a_Donation->setText(QCoreApplication::translate("MainWindow", "Make a Donation...", nullptr));
        groupBox->setTitle(QCoreApplication::translate("MainWindow", "Input ", nullptr));
#if QT_CONFIG(tooltip)
        inputFileOptBtn->setToolTip(QCoreApplication::translate("MainWindow", "If selected, input is from a file.", nullptr));
#endif // QT_CONFIG(tooltip)
        inputFileOptBtn->setText(QCoreApplication::translate("MainWindow", "File", nullptr));
#if QT_CONFIG(tooltip)
        inputDeviceOptBtn->setToolTip(QCoreApplication::translate("MainWindow", "If selected, input is from a device or GPS unit", nullptr));
#endif // QT_CONFIG(tooltip)
        inputDeviceOptBtn->setText(QCoreApplication::translate("MainWindow", "Device", nullptr));
        label->setText(QCoreApplication::translate("MainWindow", "Format", nullptr));
#if QT_CONFIG(tooltip)
        inputFormatCombo->setToolTip(QCoreApplication::translate("MainWindow", "Input data format", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        inputFileNameBrowseBtn->setToolTip(QCoreApplication::translate("MainWindow", "Browse for one or more input files. ", nullptr));
#endif // QT_CONFIG(tooltip)
        inputFileNameBrowseBtn->setText(QCoreApplication::translate("MainWindow", "File Name(s)", nullptr));
        label_2->setText(QCoreApplication::translate("MainWindow", "Device Name:", nullptr));
#if QT_CONFIG(tooltip)
        inputDeviceNameCombo->setToolTip(QCoreApplication::translate("MainWindow", "Name of port to which input device is connected", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        inputOptionsBtn->setToolTip(QCoreApplication::translate("MainWindow", "Options for the selected input format. ", nullptr));
#endif // QT_CONFIG(tooltip)
        inputOptionsBtn->setText(QCoreApplication::translate("MainWindow", "Options", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("MainWindow", "Translation Options", nullptr));
        wayPtLabel->setText(QString());
#if QT_CONFIG(tooltip)
        xlateWayPtsCk->setToolTip(QCoreApplication::translate("MainWindow", "If selected, translate waypoints.", nullptr));
#endif // QT_CONFIG(tooltip)
        xlateWayPtsCk->setText(QCoreApplication::translate("MainWindow", "Waypoints", nullptr));
        routeLabel->setText(QString());
#if QT_CONFIG(tooltip)
        xlateRoutesCk->setToolTip(QCoreApplication::translate("MainWindow", "If selected, translate routes.", nullptr));
#endif // QT_CONFIG(tooltip)
        xlateRoutesCk->setText(QCoreApplication::translate("MainWindow", "Routes", nullptr));
        trackLabel->setText(QString());
#if QT_CONFIG(tooltip)
        xlateTracksCk->setToolTip(QCoreApplication::translate("MainWindow", "If selected, translate tracks.", nullptr));
#endif // QT_CONFIG(tooltip)
        xlateTracksCk->setText(QCoreApplication::translate("MainWindow", "Tracks", nullptr));
        filterStatus->setText(QString());
#if QT_CONFIG(tooltip)
        xlateFiltersBtn->setToolTip(QCoreApplication::translate("MainWindow", "Data Filters between input and output", nullptr));
#endif // QT_CONFIG(tooltip)
        xlateFiltersBtn->setText(QCoreApplication::translate("MainWindow", "Filters", nullptr));
#if QT_CONFIG(tooltip)
        moreOptionButton->setToolTip(QCoreApplication::translate("MainWindow", "More translation options. ", nullptr));
#endif // QT_CONFIG(tooltip)
        moreOptionButton->setText(QCoreApplication::translate("MainWindow", "More Options", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("MainWindow", "Output", nullptr));
#if QT_CONFIG(tooltip)
        outputFileOptBtn->setToolTip(QCoreApplication::translate("MainWindow", "If selected, output is to a file. ", nullptr));
#endif // QT_CONFIG(tooltip)
        outputFileOptBtn->setText(QCoreApplication::translate("MainWindow", "File", nullptr));
#if QT_CONFIG(tooltip)
        outputDeviceOptBtn->setToolTip(QCoreApplication::translate("MainWindow", "If selected, output is to a device or GPS unit", nullptr));
#endif // QT_CONFIG(tooltip)
        outputDeviceOptBtn->setText(QCoreApplication::translate("MainWindow", "Device", nullptr));
        label_4->setText(QCoreApplication::translate("MainWindow", "Format", nullptr));
#if QT_CONFIG(tooltip)
        outputFormatCombo->setToolTip(QCoreApplication::translate("MainWindow", "Output data format.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        outputFileNameBrowseBtn->setToolTip(QCoreApplication::translate("MainWindow", "Browse for an output file name. ", nullptr));
#endif // QT_CONFIG(tooltip)
        outputFileNameBrowseBtn->setText(QCoreApplication::translate("MainWindow", "File Name", nullptr));
        label_6->setText(QCoreApplication::translate("MainWindow", "Device Name:", nullptr));
#if QT_CONFIG(tooltip)
        outputDeviceNameCombo->setToolTip(QCoreApplication::translate("MainWindow", "Name of port to which output device is connected", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        outputOptionsBtn->setToolTip(QCoreApplication::translate("MainWindow", "Options for the selected output format. ", nullptr));
#endif // QT_CONFIG(tooltip)
        outputOptionsBtn->setText(QCoreApplication::translate("MainWindow", "Options", nullptr));
#if QT_CONFIG(tooltip)
        outputWindow->setToolTip(QCoreApplication::translate("MainWindow", "Output of GPSBabel translation process. ", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonBox->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        menuFile->setTitle(QCoreApplication::translate("MainWindow", "File", nullptr));
        menuHelp->setTitle(QCoreApplication::translate("MainWindow", "Help", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINUI_H
