/********************************************************************************
** Form generated from reading UI file 'preferences.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFERENCES_H
#define UI_PREFERENCES_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Preferences
{
public:
    QFormLayout *formLayout;
    QTabWidget *tabWidget;
    QWidget *general_tab;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *startupCheck;
    QCheckBox *reportStatisticsCheck;
    QCheckBox *ignoreVersionMismatchCheck;
    QWidget *format_tab;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QListWidget *enabledFormatsList;
    QVBoxLayout *verticalLayout_3;
    QPushButton *enableAllButton;
    QPushButton *disableAllButton;
    QDialogButtonBox *buttonBox;
    QSpacerItem *horizontalSpacer;

    void setupUi(QDialog *Preferences)
    {
        if (Preferences->objectName().isEmpty())
            Preferences->setObjectName("Preferences");
        Preferences->resize(494, 327);
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Preferences->sizePolicy().hasHeightForWidth());
        Preferences->setSizePolicy(sizePolicy);
        formLayout = new QFormLayout(Preferences);
        formLayout->setObjectName("formLayout");
        tabWidget = new QTabWidget(Preferences);
        tabWidget->setObjectName("tabWidget");
        general_tab = new QWidget();
        general_tab->setObjectName("general_tab");
        layoutWidget = new QWidget(general_tab);
        layoutWidget->setObjectName("layoutWidget");
        layoutWidget->setGeometry(QRect(23, 18, 387, 62));
        verticalLayout_4 = new QVBoxLayout(layoutWidget);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        startupCheck = new QCheckBox(layoutWidget);
        startupCheck->setObjectName("startupCheck");

        verticalLayout_4->addWidget(startupCheck);

        reportStatisticsCheck = new QCheckBox(layoutWidget);
        reportStatisticsCheck->setObjectName("reportStatisticsCheck");

        verticalLayout_4->addWidget(reportStatisticsCheck);

        ignoreVersionMismatchCheck = new QCheckBox(layoutWidget);
        ignoreVersionMismatchCheck->setObjectName("ignoreVersionMismatchCheck");

        verticalLayout_4->addWidget(ignoreVersionMismatchCheck);

        tabWidget->addTab(general_tab, QString());
        format_tab = new QWidget();
        format_tab->setObjectName("format_tab");
        horizontalLayout = new QHBoxLayout(format_tab);
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(format_tab);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        enabledFormatsList = new QListWidget(format_tab);
        enabledFormatsList->setObjectName("enabledFormatsList");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(enabledFormatsList->sizePolicy().hasHeightForWidth());
        enabledFormatsList->setSizePolicy(sizePolicy1);
        enabledFormatsList->setSortingEnabled(false);

        verticalLayout->addWidget(enabledFormatsList);


        horizontalLayout->addLayout(verticalLayout);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        enableAllButton = new QPushButton(format_tab);
        enableAllButton->setObjectName("enableAllButton");

        verticalLayout_3->addWidget(enableAllButton);

        disableAllButton = new QPushButton(format_tab);
        disableAllButton->setObjectName("disableAllButton");

        verticalLayout_3->addWidget(disableAllButton);


        horizontalLayout->addLayout(verticalLayout_3);

        tabWidget->addTab(format_tab, QString());

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, tabWidget);

        buttonBox = new QDialogButtonBox(Preferences);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, buttonBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        formLayout->setItem(0, QFormLayout::ItemRole::FieldRole, horizontalSpacer);


        retranslateUi(Preferences);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Preferences);
    } // setupUi

    void retranslateUi(QDialog *Preferences)
    {
        Preferences->setWindowTitle(QCoreApplication::translate("Preferences", "GPSBabel Preferences", nullptr));
        startupCheck->setText(QCoreApplication::translate("Preferences", "Check for newer version on start.", nullptr));
        reportStatisticsCheck->setText(QCoreApplication::translate("Preferences", "Anonymously report usage data.", nullptr));
        ignoreVersionMismatchCheck->setText(QCoreApplication::translate("Preferences", "Ignore mismatch between command line and GUI version.", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(general_tab), QCoreApplication::translate("Preferences", "General", nullptr));
        label->setText(QCoreApplication::translate("Preferences", "Enabled Formats", nullptr));
        enableAllButton->setText(QCoreApplication::translate("Preferences", "Enable All", nullptr));
        disableAllButton->setText(QCoreApplication::translate("Preferences", "Disable All", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(format_tab), QCoreApplication::translate("Preferences", "Formats", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Preferences: public Ui_Preferences {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFERENCES_H
