/********************************************************************************
** Form generated from reading UI file 'advui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ADVUI_H
#define UI_ADVUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_AdvUi
{
public:
    QVBoxLayout *verticalLayout;
    QCheckBox *synthShortNames;
    QCheckBox *previewGmap;
    QHBoxLayout *horizontalLayout;
    QComboBox *debugCombo;
    QLabel *label;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *formatButton;
    QSpacerItem *horizontalSpacer_2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *AdvUi)
    {
        if (AdvUi->objectName().isEmpty())
            AdvUi->setObjectName("AdvUi");
        AdvUi->resize(288, 196);
        verticalLayout = new QVBoxLayout(AdvUi);
        verticalLayout->setObjectName("verticalLayout");
        synthShortNames = new QCheckBox(AdvUi);
        synthShortNames->setObjectName("synthShortNames");

        verticalLayout->addWidget(synthShortNames);

        previewGmap = new QCheckBox(AdvUi);
        previewGmap->setObjectName("previewGmap");

        verticalLayout->addWidget(previewGmap);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        debugCombo = new QComboBox(AdvUi);
        debugCombo->addItem(QString());
        debugCombo->addItem(QString());
        debugCombo->addItem(QString());
        debugCombo->addItem(QString());
        debugCombo->addItem(QString());
        debugCombo->addItem(QString());
        debugCombo->addItem(QString());
        debugCombo->addItem(QString());
        debugCombo->addItem(QString());
        debugCombo->addItem(QString());
        debugCombo->addItem(QString());
        debugCombo->setObjectName("debugCombo");

        horizontalLayout->addWidget(debugCombo);

        label = new QLabel(AdvUi);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        formatButton = new QPushButton(AdvUi);
        formatButton->setObjectName("formatButton");

        horizontalLayout_2->addWidget(formatButton);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout_2);

        buttonBox = new QDialogButtonBox(AdvUi);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(AdvUi);

        QMetaObject::connectSlotsByName(AdvUi);
    } // setupUi

    void retranslateUi(QDialog *AdvUi)
    {
        AdvUi->setWindowTitle(QCoreApplication::translate("AdvUi", "Global Options", nullptr));
#if QT_CONFIG(tooltip)
        synthShortNames->setToolTip(QCoreApplication::translate("AdvUi", "Create smart shortened names. ", nullptr));
#endif // QT_CONFIG(tooltip)
        synthShortNames->setText(QCoreApplication::translate("AdvUi", "Synthesize short names", nullptr));
        previewGmap->setText(QCoreApplication::translate("AdvUi", "Preview in Google Maps", nullptr));
        debugCombo->setItemText(0, QCoreApplication::translate("AdvUi", "None", nullptr));
        debugCombo->setItemText(1, QCoreApplication::translate("AdvUi", "0", nullptr));
        debugCombo->setItemText(2, QCoreApplication::translate("AdvUi", "1", nullptr));
        debugCombo->setItemText(3, QCoreApplication::translate("AdvUi", "2", nullptr));
        debugCombo->setItemText(4, QCoreApplication::translate("AdvUi", "3", nullptr));
        debugCombo->setItemText(5, QCoreApplication::translate("AdvUi", "4", nullptr));
        debugCombo->setItemText(6, QCoreApplication::translate("AdvUi", "5", nullptr));
        debugCombo->setItemText(7, QCoreApplication::translate("AdvUi", "6", nullptr));
        debugCombo->setItemText(8, QCoreApplication::translate("AdvUi", "7", nullptr));
        debugCombo->setItemText(9, QCoreApplication::translate("AdvUi", "8", nullptr));
        debugCombo->setItemText(10, QCoreApplication::translate("AdvUi", "9", nullptr));

#if QT_CONFIG(tooltip)
        debugCombo->setToolTip(QCoreApplication::translate("AdvUi", "Debugging diagnostics.  \n"
"Higher number provides more detailed diagnostics.", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("AdvUi", "Debugging Diagnostics", nullptr));
#if QT_CONFIG(tooltip)
        formatButton->setToolTip(QCoreApplication::translate("AdvUi", "Set all format input/output options to default values", nullptr));
#endif // QT_CONFIG(tooltip)
        formatButton->setText(QCoreApplication::translate("AdvUi", "Default Format Options", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AdvUi: public Ui_AdvUi {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ADVUI_H
